@@ -28,9 +28,8 @@
 
 #include "drivers/lirc.h"
 
-int bte_decode(struct ir_remote *remote,
-	       ir_code *prep,ir_code *codep,ir_code *postp,
-	       int *repeat_flagp,lirc_t *remaining_gapp);
+int bte_decode(struct ir_remote *remote, ir_code * prep, ir_code * codep, ir_code * postp, int *repeat_flagp,
+	       lirc_t * min_remaining_gapp, lirc_t * max_remaining_gapp);
 int bte_init(void);
 int bte_deinit(void);
 char *bte_rec(struct ir_remote *remotes);
