@@ -1,4 +1,4 @@
-/*      $Id: hw_default.h,v 5.11 2007/07/29 18:20:07 lirc Exp $      */
+/*      $Id: hw_default.h,v 5.12 2010/01/16 17:17:42 lirc Exp $      */
 
 /****************************************************************************
  ** hw_default.h ************************************************************
@@ -15,15 +15,12 @@
 
 #include "ir_remote.h"
 
-inline lirc_t time_left(struct timeval *current,struct timeval *last,
-			lirc_t gap);
-
 int default_init(void);
 int default_config(struct ir_remote *remotes);
 int default_deinit(void);
-int default_send(struct ir_remote *remote,struct ir_ncode *code);
+int default_send(struct ir_remote *remote, struct ir_ncode *code);
 char *default_rec(struct ir_remote *remotes);
 int default_ioctl(unsigned int cmd, void *arg);
-lirc_t default_readdata (lirc_t timeout);
+lirc_t default_readdata(lirc_t timeout);
 
 #endif
