@@ -1,4 +1,4 @@
-/*     $Id: hw_mp3anywhere.h,v 5.1 2003/02/16 19:06:56 lirc Exp $     */
+/*     $Id: hw_mp3anywhere.h,v 5.2 2007/07/29 18:20:08 lirc Exp $     */
 
 /****************************************************************************
  ** hw_mp3anywhere.h ********************************************************
@@ -17,9 +17,8 @@
 
 #include "drivers/lirc.h"
 
-int mp3anywhere_decode(struct ir_remote *remote,
-		  ir_code *prep,ir_code *codep,ir_code *postp,
-		  int *repeat_flagp,lirc_t *remaining_gapp);
+int mp3anywhere_decode(struct ir_remote *remote, ir_code * prep, ir_code * codep, ir_code * postp, int *repeat_flagp,
+		       lirc_t * min_remaining_gapp, lirc_t * max_remaining_gapp);
 int mp3anywhere_init(void);
 int mp3anywhere_deinit(void);
 char *mp3anywhere_rec(struct ir_remote *remotes);
