@@ -14,11 +14,8 @@
 
 #include "drivers/lirc.h"
 
-int silitek_decode(struct ir_remote *remote,
-		   ir_code *prep,ir_code *codep,ir_code *postp,
-		   int *repeat_flagp,
-		   lirc_t *min_remaining_gapp,
-		   lirc_t *max_remaining_gapp);
+int silitek_decode(struct ir_remote *remote, ir_code * prep, ir_code * codep, ir_code * postp, int *repeat_flagp,
+		   lirc_t * min_remaining_gapp, lirc_t * max_remaining_gapp);
 int silitek_init(void);
 int silitek_deinit(void);
 char *silitek_rec(struct ir_remote *remotes);
