@@ -1,4 +1,4 @@
-/*      $Id: hw_uirt2_common.c,v 5.2 2003/10/26 14:52:45 lirc Exp $   */
+/*      $Id: hw_uirt2_common.c,v 5.7 2010/05/25 19:12:26 lirc Exp $   */
 
 /****************************************************************************
  ** hw_uirt2_common.c *******************************************************
@@ -38,6 +38,7 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <stdarg.h>
+#include <errno.h>
 #include "serial.h"
 #include "lircd.h"
 #include "hw_uirt2_common.h"
@@ -54,6 +55,7 @@ LOGPRINTF(1, "time: %s %li %li", #a, (a)->tv_sec, (a)->tv_usec)
 struct tag_uirt2_t {
 	int fd;
 	int flags;
+	int version;
 
 	struct timeval pre_delay;
 	struct timeval pre_time;
@@ -63,9 +65,45 @@ struct tag_uirt2_t {
 const int unit = UIRT2_UNIT;
 //static int debug = 3;
 
+static ssize_t readagain(int fd, void *buf, size_t count)
+{
+	ssize_t rc;
+	size_t pos = 0;
+	struct timeval timeout = {.tv_sec = 0,.tv_usec = 200000 };
+	fd_set fds;
+
+	rc = read(fd, buf, count);
+
+	if (rc > 0) {
+		pos += rc;
+	}
+
+	while ((rc == -1 && errno == EAGAIN) || (rc >= 0 && pos < count)) {
+		FD_ZERO(&fds);
+		FD_SET(fd, &fds);
+
+		rc = select(fd + 1, &fds, NULL, NULL, &timeout);
+
+		if (rc == 0) {
+			/* timeout */
+			break;
+		} else if (rc == -1) {
+			/* continue for EAGAIN case */
+			continue;
+		}
+
+		rc = read(fd, ((char *)buf) + pos, count - pos);
+
+		if (rc > 0) {
+			pos += rc;
+		}
+
+	}
+	return (pos == 0) ? -1 : pos;
+}
 
 #ifdef DEBUG
-static void hexdump(byte_t *buf, int len)
+static void hexdump(byte_t * buf, int len)
 {
 	int i;
 	char str[200];
@@ -89,8 +127,8 @@ static void hexdump(byte_t *buf, int len)
 }
 #endif /* DEBUG */
 
-
-static int mywaitfordata(uirt2_t *dev, long usec) {
+static int mywaitfordata(uirt2_t * dev, long usec)
+{
 	int fd = dev->fd;
 	fd_set fds;
 	int maxfd = fd;
@@ -98,10 +136,10 @@ static int mywaitfordata(uirt2_t *dev, long usec) {
 	struct timeval tv;
 
 	FD_ZERO(&fds);
-	FD_SET(fd,&fds);
+	FD_SET(fd, &fds);
 
-	tv.tv_sec = 0;
-	tv.tv_usec = usec;
+	tv.tv_sec = usec / 1000000;
+	tv.tv_usec = usec % 1000000;
 
 	ret = select(maxfd + 1, &fds, NULL, NULL, &tv);
 
@@ -112,8 +150,21 @@ static int mywaitfordata(uirt2_t *dev, long usec) {
 	}
 }
 
+static int uirt2_readflush(uirt2_t * dev, long timeout)
+{
+	int res;
+	char c;
 
-static byte_t checksum(byte_t *data, int len)
+	while (mywaitfordata(dev, timeout) > 0) {
+		res = readagain(dev->fd, &c, 1);
+		if (res < 1) {
+			return -1;
+		}
+	}
+	return 0;
+}
+
+static byte_t checksum(byte_t * data, int len)
 {
 	int check = 0;
 	int i;
@@ -125,8 +176,7 @@ static byte_t checksum(byte_t *data, int len)
 	return check & 0xff;
 }
 
-
-static int command_ext(uirt2_t *dev, const byte_t *in, byte_t *out)
+static int command_ext(uirt2_t * dev, const byte_t * in, byte_t * out)
 {
 	byte_t tmp[1024];
 	int res;
@@ -137,7 +187,7 @@ static int command_ext(uirt2_t *dev, const byte_t *in, byte_t *out)
 
 	tmp[len + 1] = checksum(tmp, len + 1) & 0xff;
 
-	if (dev->pre_delay.tv_sec >= 0 && dev->pre_delay.tv_usec > 0) {
+	if (timerisset(&dev->pre_delay)) {
 		struct timeval cur;
 		struct timeval diff;
 		struct timeval delay;
@@ -146,19 +196,20 @@ static int command_ext(uirt2_t *dev, const byte_t *in, byte_t *out)
 		timersub(&cur, &dev->pre_time, &diff);
 		PRINT_TIME(&diff);
 
-		timersub(&dev->pre_delay, &diff, &delay);
-		PRINT_TIME(&delay);
+		if (timercmp(&dev->pre_delay, &diff, >)) {
+			timersub(&dev->pre_delay, &diff, &delay);
+			PRINT_TIME(&delay);
 
-		if (delay.tv_sec >= 0 && delay.tv_usec > 0) {
-			LOGPRINTF(1, "udelay %lu %lu", 
-				  delay.tv_sec, delay.tv_usec);
+			LOGPRINTF(1, "udelay %lu %lu", delay.tv_sec, delay.tv_usec);
 			sleep(delay.tv_sec);
 			usleep(delay.tv_usec);
 		}
-	
-		dev->pre_delay.tv_sec = -1;
+
+		timerclear(&dev->pre_delay);
 	}
 
+	uirt2_readflush(dev, 0);
+
 	LOGPRINTF(1, "writing command %02x", buf[0]);
 
 	HEXDUMP(tmp, len + 2);
@@ -171,12 +222,12 @@ static int command_ext(uirt2_t *dev, const byte_t *in, byte_t *out)
 
 	LOGPRINTF(1, "wrote %d", res);
 
-	if (!mywaitfordata(dev, (long) 1000000)) {
+	if (!mywaitfordata(dev, (long)1000000)) {
 		logprintf(LOG_ERR, "uirt2_raw: did not receive results");
 		return -1;
 	}
 
-	res = read(dev->fd, out + 1, out[0]);
+	res = readagain(dev->fd, out + 1, out[0]);
 
 	if (res < out[0]) {
 		logprintf(LOG_ERR, "uirt2_raw: couldn't read command result");
@@ -195,17 +246,16 @@ static int command_ext(uirt2_t *dev, const byte_t *in, byte_t *out)
 			return -1;
 		}
 	}
-    
+
 	return 0;
 }
 
-
-static int command(uirt2_t *dev, const byte_t *buf, int len)
+static int command(uirt2_t * dev, const byte_t * buf, int len)
 {
 	byte_t in[1024];
 	byte_t out[2];
 
-	memcpy(in + 1, buf, len+1);
+	memcpy(in + 1, buf, len + 1);
 	in[0] = len;
 	out[0] = 1;
 
@@ -216,13 +266,12 @@ static int command(uirt2_t *dev, const byte_t *buf, int len)
 	return out[1] < UIRT2_CSERROR;
 }
 
-
-static unsigned long calc_bits_length(remstruct1_t *buf)
+static __u32 calc_bits_length(remstruct1_data_t * buf)
 {
 	int i;
 	byte_t b = 0;
-	unsigned long len = 0;
-    
+	__u32 len = 0;
+
 	for (i = 0; i < buf->bBits; i++) {
 		int bit;
 
@@ -239,97 +288,89 @@ static unsigned long calc_bits_length(remstruct1_t *buf)
 				len += buf->bOff1;
 			} else {
 				len += buf->bOff0;
-			}		
+			}
 		} else {
 			// Even
 			if (bit) {
 				len += buf->bOn1;
 			} else {
 				len += buf->bOn0;
-			}		
+			}
 		}
 	}
 
 	return unit * len;
 }
 
-
-static unsigned long calc_struct1_length(remstruct1_t *buf)
+static __u32 calc_struct1_length(int repeat, remstruct1_data_t * buf)
 {
-	int repeat = buf->bCmd & 0x1f;
 	int bISDly = unit * (buf->bISDlyLo + 256 * buf->bISDlyHi);
 	int bHdr = unit * (buf->bHdr1 + buf->bHdr0);
-	unsigned long bBitLength = calc_bits_length(buf);
+	__u32 bBitLength = calc_bits_length(buf);
 
 	LOGPRINTF(1, "bBitLength %lu repeat %d", bBitLength, repeat);
 
 	return (repeat + 1) * (bISDly + bHdr + bBitLength);
 }
 
-
-
 /*
  * Exported functions 
  */
 
 uirt2_t *uirt2_init(int fd)
 {
-	uirt2_t *dev = (uirt2_t *)malloc(sizeof(uirt2_t));
-	int version = 0;
+	uirt2_t *dev = (uirt2_t *) malloc(sizeof(uirt2_t));
 
-	if(dev == NULL)
-	{
+	if (dev == NULL) {
 		logprintf(LOG_ERR, "uirt2_raw: out of memory");
 		return NULL;
 	}
-	
-        memset(dev, 0, sizeof(uirt2_t));
 
+	memset(dev, 0, sizeof(uirt2_t));
+
+	timerclear(&dev->pre_time);
 	dev->new_signal = 1;
 	dev->flags = UIRT2_MODE_UIR;
 	dev->fd = fd;
 
-	if(uirt2_getversion(dev, &version) < 0) {
+	uirt2_readflush(dev, 200000);
+
+	if (uirt2_getversion(dev, &dev->version) < 0) {
 		free(dev);
 		return NULL;
 	}
 
-	if(version < 0x0104) {
+	if (dev->version < 0x0104) {
 		logprintf(LOG_WARNING, "uirt2_raw: Old UIRT hardware");
 	} else {
-		logprintf(LOG_INFO, "uirt2_raw: UIRT version %04x ok", 
-			  version);
+		logprintf(LOG_INFO, "uirt2_raw: UIRT version %04x ok", dev->version);
 	}
 
 	return dev;
 }
 
-
-int uirt2_uninit(uirt2_t *dev)
+int uirt2_uninit(uirt2_t * dev)
 {
 	free(dev);
 	return 0;
 }
 
-
-int uirt2_getmode(uirt2_t *dev)
+int uirt2_getmode(uirt2_t * dev)
 {
 	return (dev->flags & UIRT2_MODE_MASK);
 }
 
-
-int uirt2_setmode(uirt2_t *dev, int mode)
+int uirt2_setmode(uirt2_t * dev, int mode)
 {
 	byte_t buf[20];
 	byte_t cmd;
 
-	if (uirt2_getmode(dev) == mode)
-	{
+	if (uirt2_getmode(dev) == mode) {
 		LOGPRINTF(1, "uirt2_setmode: already in requested mode");
 		return 0;
 	}
 
-	switch(mode) {
+	switch (mode) {
 	case UIRT2_MODE_UIR:
 		cmd = UIRT2_SETMODEUIR;
 		break;
@@ -345,7 +386,7 @@ int uirt2_setmode(uirt2_t *dev, int mode)
 	}
 
 	buf[0] = cmd;
-	
+
 	if (command(dev, buf, 0) < 0) {
 		logprintf(LOG_ERR, "uirt2_raw: setmode failed");
 		return -1;
@@ -355,48 +396,64 @@ int uirt2_setmode(uirt2_t *dev, int mode)
 	return 0;
 }
 
-
-int uirt2_setmodeuir(uirt2_t *dev)
+int uirt2_setmodeuir(uirt2_t * dev)
 {
 	return uirt2_setmode(dev, UIRT2_MODE_UIR);
 }
 
-
-int uirt2_setmoderaw(uirt2_t *dev)
+int uirt2_setmoderaw(uirt2_t * dev)
 {
 	return uirt2_setmode(dev, UIRT2_MODE_RAW);
 }
-    
 
-int uirt2_setmodestruc(uirt2_t *dev)
+int uirt2_setmodestruc(uirt2_t * dev)
 {
 	return uirt2_setmode(dev, UIRT2_MODE_STRUC);
 }
 
-
-int uirt2_getversion(uirt2_t *dev, int *version)
+int uirt2_getversion(uirt2_t * dev, int *version)
 {
 	byte_t out[20];
 	byte_t in[20];
 
+	if (dev->version != 0) {
+		*version = dev->version;
+		return 0;
+	}
+
 	in[0] = 0;
 	in[1] = UIRT2_GETVERSION;
 	out[0] = 3;
-	
-	if (command_ext(dev, in, out) < 0) {
-		return -1;
+
+	if (command_ext(dev, in, out) >= 0) {
+		*version = out[2] + (out[1] << 8);
+		return 0;
 	}
 
-	*version = out[2] + (out[1] << 8);
-	return 0;
+	/* 
+	 * Ok, that command didn't work.  Maybe we're 
+	 * dealing with a newer version of the UIRT2 
+	 * protocol, which sends extended information when 
+	 * the version is requested.
+	 */
+	LOGPRINTF(0, "uirt2: detection of uirt2 failed");
+	LOGPRINTF(0, "uirt2: trying to detect newer uirt firmware");
+	uirt2_readflush(dev, 200000);
+
+	out[0] = 8;
+	if (command_ext(dev, in, out) >= 0) {
+		*version = out[2] + (out[1] << 8);
+		return 0;
+	}
+
+	return -1;
 }
 
-
-int uirt2_getgpiocaps(uirt2_t *dev, int *slots, byte_t masks[4])
+int uirt2_getgpiocaps(uirt2_t * dev, int *slots, byte_t masks[4])
 {
 	byte_t in[3];
 	byte_t out[6];
-	
+
 	in[0] = 1;
 	in[1] = UIRT2_GETGPIOCAPS;
 	in[2] = 1;
@@ -412,9 +469,7 @@ int uirt2_getgpiocaps(uirt2_t *dev, int *slots, byte_t masks[4])
 	return 0;
 }
 
-
-int uirt2_getgpiocfg(uirt2_t *dev, int slot, uirt2_code_t code,
-		     int *action, int *duration)
+int uirt2_getgpiocfg(uirt2_t * dev, int slot, uirt2_code_t code, int *action, int *duration)
 {
 	byte_t in[4];
 	byte_t out[10];
@@ -424,8 +479,8 @@ int uirt2_getgpiocfg(uirt2_t *dev, int slot, uirt2_code_t code,
 	in[2] = 2;
 	in[3] = slot;
 	out[0] = 9;
-	
-	if(command_ext(dev, in, out) < 0) {
+
+	if (command_ext(dev, in, out) < 0) {
 		return -1;
 	}
 
@@ -435,9 +490,7 @@ int uirt2_getgpiocfg(uirt2_t *dev, int slot, uirt2_code_t code,
 	return 0;
 }
 
-
-int uirt2_setgpiocfg(uirt2_t *dev, int slot, uirt2_code_t code,
-		     int action, int duration)
+int uirt2_setgpiocfg(uirt2_t * dev, int slot, uirt2_code_t code, int action, int duration)
 {
 	byte_t in[12];
 
@@ -445,7 +498,7 @@ int uirt2_setgpiocfg(uirt2_t *dev, int slot, uirt2_code_t code,
 	in[1] = UIRT2_SETGPIOCFG;
 	in[2] = 4 + UIRT2_CODE_SIZE;
 	in[3] = slot;
-	
+
 	memcpy(in + 4, code, UIRT2_CODE_SIZE);
 
 	in[10] = action;
@@ -454,8 +507,7 @@ int uirt2_setgpiocfg(uirt2_t *dev, int slot, uirt2_code_t code,
 	return command(dev, in + 1, in[0]);
 }
 
-
-int uirt2_getgpio(uirt2_t *dev, byte_t ports[4])
+int uirt2_getgpio(uirt2_t * dev, byte_t ports[4])
 {
 	byte_t in[3];
 	byte_t out[6];
@@ -464,8 +516,8 @@ int uirt2_getgpio(uirt2_t *dev, byte_t ports[4])
 	in[1] = UIRT2_GETGPIO;
 	in[2] = 1;
 	out[0] = 5;
-	
-	if(command_ext(dev, in, out) < 0) {
+
+	if (command_ext(dev, in, out) < 0) {
 		return -1;
 	}
 
@@ -473,8 +525,7 @@ int uirt2_getgpio(uirt2_t *dev, byte_t ports[4])
 	return 0;
 }
 
-
-int uirt2_setgpio(uirt2_t *dev, int action, int duration)
+int uirt2_setgpio(uirt2_t * dev, int action, int duration)
 {
 	byte_t buf[20];
 
@@ -486,8 +537,7 @@ int uirt2_setgpio(uirt2_t *dev, int action, int duration)
 	return command(dev, buf, 3);
 }
 
-
-int uirt2_refreshgpio(uirt2_t *dev)
+int uirt2_refreshgpio(uirt2_t * dev)
 {
 	byte_t buf[2];
 
@@ -497,8 +547,7 @@ int uirt2_refreshgpio(uirt2_t *dev)
 	return command(dev, buf, 1);
 }
 
-
-int uirt2_read_uir(uirt2_t *dev, byte_t *buf, int length)
+int uirt2_read_uir(uirt2_t * dev, byte_t * buf, int length)
 {
 	int pos = 0;
 	int res;
@@ -509,7 +558,7 @@ int uirt2_read_uir(uirt2_t *dev, byte_t *buf, int length)
 	}
 
 	while (1) {
-		res = read(dev->fd, buf + pos, 1);
+		res = readagain(dev->fd, buf + pos, 1);
 
 		if (res == -1) {
 			return pos;
@@ -521,12 +570,11 @@ int uirt2_read_uir(uirt2_t *dev, byte_t *buf, int length)
 			break;
 		}
 	}
-    
+
 	return pos;
 }
 
-
-lirc_t uirt2_read_raw(uirt2_t *dev, lirc_t timeout)
+lirc_t uirt2_read_raw(uirt2_t * dev, lirc_t timeout)
 {
 	lirc_t data;
 	static int pulse = 0;
@@ -543,7 +591,7 @@ lirc_t uirt2_read_raw(uirt2_t *dev, lirc_t timeout)
 		if (!waitfordata(timeout))
 			return 0;
 
-		res = read(dev->fd, &b, 1);
+		res = readagain(dev->fd, &b, 1);
 
 		if (res == -1) {
 			return 0;
@@ -562,7 +610,7 @@ lirc_t uirt2_read_raw(uirt2_t *dev, lirc_t timeout)
 			isdly[0] = b;
 			LOGPRINTF(1, "dev->new_signal");
 
-			res = read(dev->fd, &isdly[1], 1);
+			res = readagain(dev->fd, &isdly[1], 1);
 
 			if (res == -1) {
 				return 0;
@@ -573,10 +621,12 @@ lirc_t uirt2_read_raw(uirt2_t *dev, lirc_t timeout)
 			dev->new_signal = 0;
 		} else {
 			data = UIRT2_UNIT * b;
+			if (data == 0)
+				data = 1;
 			if (pulse) {
 				data = data | PULSE_BIT;
 			}
-		
+
 			pulse = !pulse;
 		}
 
@@ -586,37 +636,60 @@ lirc_t uirt2_read_raw(uirt2_t *dev, lirc_t timeout)
 	return 0;
 }
 
-
-int uirt2_send_raw(uirt2_t *dev, byte_t *buf, int length)
+int uirt2_send_raw(uirt2_t * dev, byte_t * buf, int length)
 {
 	byte_t tmp[1024];
 
 	tmp[0] = UIRT2_DOTXRAW;
 	tmp[1] = length + 1;
 	memcpy(tmp + 2, buf, length);
-	
+
 	return command(dev, tmp, length + 1);
 }
 
-
-int uirt2_send_struct1(uirt2_t *dev, remstruct1_t *buf)
+int uirt2_send_struct1(uirt2_t * dev, int freq, int bRepeatCount, remstruct1_data_t * buf)
 {
 	int res;
-	unsigned long delay;
+	__u32 delay;
+	remstruct1_t rem;
+	remstruct1_ext_t rem_ext;
 
-	res = command(dev, (byte_t *)buf, sizeof(remstruct1_t) - 2);
-	delay = calc_struct1_length(buf);
+	if (dev->version >= 0x0905) {
+		byte_t tmp[2 + sizeof(remstruct1_ext_t)];
+
+		if (freq == 0 || ((5000000 / freq) + 1) / 2 >= 0x80) {
+			rem_ext.bFrequency = 0x80;
+		} else {
+			rem_ext.bFrequency = ((5000000 / freq) + 1) / 2;
+		}
+		rem_ext.bRepeatCount = bRepeatCount;
+		memcpy(&rem_ext.data, buf, sizeof(*buf));
+
+		tmp[0] = 0x37;
+		tmp[1] = sizeof(rem_ext) + 1;
+
+		memcpy(tmp + 2, &rem_ext, sizeof(rem_ext));
+		res = command(dev, tmp, sizeof(rem_ext) + 1);
+	} else {
+		if (bRepeatCount > 0x1f) {
+			rem.bCmd = uirt2_calc_freq(freq) + 0x1f;
+		} else {
+			rem.bCmd = uirt2_calc_freq(freq) + bRepeatCount;
+		}
+		memcpy(&rem.data, buf, sizeof(*buf));
+
+		res = command(dev, (byte_t *) & rem, sizeof(rem) - 2);
+	}
+	delay = calc_struct1_length(bRepeatCount, buf);
 	gettimeofday(&dev->pre_time, NULL);
 	dev->pre_delay.tv_sec = delay / 1000000;
 	dev->pre_delay.tv_usec = delay % 1000000;
 
-	LOGPRINTF(1, "set dev->pre_delay %lu %lu",
-		  dev->pre_delay.tv_sec, dev->pre_delay.tv_usec);
+	LOGPRINTF(1, "set dev->pre_delay %lu %lu", dev->pre_delay.tv_sec, dev->pre_delay.tv_usec);
 
 	return res;
 }
 
-
 int uirt2_calc_freq(int freq)
 {
 	if (freq > 39000) {
