@@ -1,4 +1,3 @@
-/*      $Id: lirc_parallel.c,v 5.56 2010/08/27 05:06:06 jarodwilson Exp $      */
 /*
  * lirc_parallel.c
  *
@@ -28,11 +27,6 @@
 # include <config.h>
 #endif
 #include <linux/version.h>
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 18)
-#error "**********************************************************"
-#error " Sorry, this driver needs kernel version 2.2.18 or higher "
-#error "**********************************************************"
-#endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33)
 #include <linux/autoconf.h>
@@ -49,27 +43,22 @@
 #include <linux/mm.h>
 #include <linux/delay.h>
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
-#include <asm/io.h>
-#include <asm/signal.h>
-#include <asm/irq.h>
-#include <asm/uaccess.h>
-#else
 #include <linux/io.h>
-#include <linux/signal.h>
 #include <linux/irq.h>
 #include <linux/uaccess.h>
-#endif
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
 #include <asm/div64.h>
-#endif
 
 #include <linux/poll.h>
 #include <linux/parport.h>
 
-#include "drivers/lirc.h"
 #include "drivers/kcompat.h"
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 35)
+#include <media/lirc.h>
+#include <media/lirc_dev.h>
+#else
+#include "drivers/lirc.h"
 #include "drivers/lirc_dev/lirc_dev.h"
+#endif
 
 #include "lirc_parallel.h"
 
@@ -97,11 +86,9 @@ unsigned int timer;
 unsigned int default_timer = LIRC_TIMER;
 #endif
 
-#define WBUF_SIZE (256)
 #define RBUF_SIZE (256) /* this must be a power of 2 larger than 1 */
 
-static lirc_t wbuf[WBUF_SIZE];
-static lirc_t rbuf[RBUF_SIZE];
+static int rbuf[RBUF_SIZE];
 
 DECLARE_WAIT_QUEUE_HEAD(lirc_wait);
 
@@ -237,7 +224,7 @@ static int lirc_claim(void)
 
 /*** interrupt handler ***/
 
-static void rbuf_write(lirc_t signal)
+static void rbuf_write(int signal)
 {
 	unsigned int nwptr;
 
@@ -264,11 +251,11 @@ static void irq_handler(void *blah)
 	static struct timeval lasttv;
 	static int init;
 	long signal;
-	lirc_t data;
+	int data;
 	unsigned int level, newlevel;
 	unsigned int timeout;
 
-	if (!MOD_IN_USE)
+	if (!is_open)
 		return;
 
 	if (!is_claimed)
@@ -276,8 +263,8 @@ static void irq_handler(void *blah)
 
 #if 0
 	/* disable interrupt */
-	disable_irq(irq);
-	out(LIRC_PORT_IRQ, in(LIRC_PORT_IRQ) & (~LP_PINTEN));
+	  disable_irq(irq);
+	  out(LIRC_PORT_IRQ, in(LIRC_PORT_IRQ) & (~LP_PINTEN));
 #endif
 	if (check_pselecd && (in(1) & LP_PSELECD))
 		return;
@@ -291,7 +278,7 @@ static void irq_handler(void *blah)
 			/* really long time */
 			data = PULSE_MASK;
 		else
-			data = (lirc_t) (signal*1000000 +
+			data = (int) (signal*1000000 +
 					 tv.tv_usec - lasttv.tv_usec +
 					 LIRC_SFH506_DELAY);
 
@@ -328,16 +315,12 @@ static void irq_handler(void *blah)
 	} while (lirc_get_signal());
 
 	if (signal != 0) {
-		/* adjust value to usecs */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
-		unsigned long long helper;
+		/* ajust value to usecs */
+		__u64 helper;
 
-		helper = ((unsigned long long) signal)*1000000;
+		helper = ((__u64) signal)*1000000;
 		do_div(helper, timer);
 		signal = (long) helper;
-#else
-		signal = (long) ((((double) signal)*1000000)/timer);
-#endif
 
 		if (signal > LIRC_SFH506_DELAY)
 			data = signal - LIRC_SFH506_DELAY;
@@ -372,7 +355,7 @@ static ssize_t lirc_read(struct file *filep, char *buf, size_t n, loff_t *ppos)
 	int count = 0;
 	DECLARE_WAITQUEUE(wait, current);
 
-	if (n % sizeof(lirc_t))
+	if (n % sizeof(int))
 		return -EINVAL;
 
 	add_wait_queue(&lirc_wait, &wait);
@@ -380,12 +363,12 @@ static ssize_t lirc_read(struct file *filep, char *buf, size_t n, loff_t *ppos)
 	while (count < n) {
 		if (rptr != wptr) {
 			if (copy_to_user(buf+count, (char *) &rbuf[rptr],
-					 sizeof(lirc_t))) {
+					 sizeof(int))) {
 				result = -EFAULT;
 				break;
 			}
 			rptr = (rptr + 1) & (RBUF_SIZE - 1);
-			count += sizeof(lirc_t);
+			count += sizeof(int);
 		} else {
 			if (filep->f_flags & O_NONBLOCK) {
 				result = -EAGAIN;
@@ -411,41 +394,39 @@ static ssize_t lirc_write(struct file *filep, const char *buf, size_t n,
 	unsigned int i;
 	unsigned int level, newlevel;
 	unsigned long flags;
-	lirc_t counttimer;
+	int counttimer;
+	int *wbuf;
+	ssize_t ret;
 
 	if (!is_claimed)
 		return -EBUSY;
 
-	if (n % sizeof(lirc_t))
+	count = n / sizeof(int);
+
+	if (n % sizeof(int) || count % 2 == 0)
 		return -EINVAL;
 
-	count = n / sizeof(lirc_t);
-
-	if (count > WBUF_SIZE || count % 2 == 0)
-		return -EINVAL;
-
-	if (copy_from_user(wbuf, buf, n))
-		return -EFAULT;
+	wbuf = memdup_user(buf, n);
+	if (IS_ERR(wbuf))
+		return PTR_ERR(wbuf);
 
 #ifdef LIRC_TIMER
 	if (timer == 0) {
 		/* try again if device is ready */
 		timer = init_lirc_timer();
-		if (timer == 0)
-			return -EIO;
+		if (timer == 0) {
+			ret = -EIO;
+			goto out;
+		}
 	}
 
 	/* adjust values from usecs */
 	for (i = 0; i < count; i++) {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
-		unsigned long long helper;
+		__u64 helper;
 
-		helper = ((unsigned long long) wbuf[i])*timer;
+		helper = ((__u64) wbuf[i])*timer;
 		do_div(helper, 1000000);
-		wbuf[i] = (lirc_t) helper;
-#else
-		wbuf[i] = (lirc_t) (((double) wbuf[i])*timer/1000000);
-#endif
+		wbuf[i] = (int) helper;
 	}
 
 	local_irq_save(flags);
@@ -462,7 +443,8 @@ static ssize_t lirc_write(struct file *filep, const char *buf, size_t n,
 			if (check_pselecd && (in(1) & LP_PSELECD)) {
 				lirc_off();
 				local_irq_restore(flags);
-				return -EIO;
+				ret = -EIO;
+				goto out;
 			}
 		} while (counttimer < wbuf[i]);
 		i++;
@@ -478,7 +460,8 @@ static ssize_t lirc_write(struct file *filep, const char *buf, size_t n,
 			level = newlevel;
 			if (check_pselecd && (in(1) & LP_PSELECD)) {
 				local_irq_restore(flags);
-				return -EIO;
+				ret = -EIO;
+				goto out;
 			}
 		} while (counttimer < wbuf[i]);
 		i++;
@@ -487,7 +470,11 @@ static ssize_t lirc_write(struct file *filep, const char *buf, size_t n,
 #else
 	/* place code that handles write without external timer here */
 #endif
-	return n;
+	ret = n;
+out:
+	kfree(wbuf);
+
+	return ret;
 }
 
 static unsigned int lirc_poll(struct file *file, poll_table *wait)
@@ -506,48 +493,48 @@ static long lirc_ioctl(struct file *filep, unsigned int cmd, unsigned long arg)
 #endif
 {
 	int result;
-	unsigned long features = LIRC_CAN_SET_TRANSMITTER_MASK |
-				 LIRC_CAN_SEND_PULSE | LIRC_CAN_REC_MODE2;
-	unsigned long mode;
-	unsigned int ivalue;
+	__u32 features = LIRC_CAN_SET_TRANSMITTER_MASK |
+			 LIRC_CAN_SEND_PULSE | LIRC_CAN_REC_MODE2;
+	__u32 mode;
+	__u32 value;
 
 	switch (cmd) {
 	case LIRC_GET_FEATURES:
-		result = put_user(features, (unsigned long *) arg);
+		result = put_user(features, (__u32 *) arg);
 		if (result)
 			return result;
 		break;
 	case LIRC_GET_SEND_MODE:
-		result = put_user(LIRC_MODE_PULSE, (unsigned long *) arg);
+		result = put_user(LIRC_MODE_PULSE, (__u32 *) arg);
 		if (result)
 			return result;
 		break;
 	case LIRC_GET_REC_MODE:
-		result = put_user(LIRC_MODE_MODE2, (unsigned long *) arg);
+		result = put_user(LIRC_MODE_MODE2, (__u32 *) arg);
 		if (result)
 			return result;
 		break;
 	case LIRC_SET_SEND_MODE:
-		result = get_user(mode, (unsigned long *) arg);
+		result = get_user(mode, (__u32 *) arg);
 		if (result)
 			return result;
 		if (mode != LIRC_MODE_PULSE)
 			return -EINVAL;
 		break;
 	case LIRC_SET_REC_MODE:
-		result = get_user(mode, (unsigned long *) arg);
+		result = get_user(mode, (__u32 *) arg);
 		if (result)
 			return result;
 		if (mode != LIRC_MODE_MODE2)
 			return -ENOSYS;
 		break;
 	case LIRC_SET_TRANSMITTER_MASK:
-		result = get_user(ivalue, (unsigned int *) arg);
+		result = get_user(value, (__u32 *) arg);
 		if (result)
 			return result;
-		if ((ivalue & LIRC_PARALLEL_TRANSMITTER_MASK) != ivalue)
+		if ((value & LIRC_PARALLEL_TRANSMITTER_MASK) != value)
 			return LIRC_PARALLEL_MAX_TRANSMITTERS;
-		tx_mask = ivalue;
+		tx_mask = value;
 		break;
 	default:
 		return -ENOIOCTLCMD;
@@ -557,7 +544,7 @@ static long lirc_ioctl(struct file *filep, unsigned int cmd, unsigned long arg)
 
 static int lirc_open(struct inode *node, struct file *filep)
 {
-	if (MOD_IN_USE || !lirc_claim())
+	if (is_open || !lirc_claim())
 		return -EBUSY;
 
 	parport_enable_irq(pport);
@@ -567,7 +554,6 @@ static int lirc_open(struct inode *node, struct file *filep)
 	wptr = 0;
 	lost_irqs = 0;
 
-	MOD_INC_USE_COUNT;
 	is_open = 1;
 	return 0;
 }
@@ -579,11 +565,10 @@ static int lirc_close(struct inode *node, struct file *filep)
 		parport_release(ppdevice);
 	}
 	is_open = 0;
-	MOD_DEC_USE_COUNT;
 	return 0;
 }
 
-static struct file_operations lirc_fops = {
+static const struct file_operations lirc_fops = {
 	.owner		= THIS_MODULE,
 	.llseek		= lirc_lseek,
 	.read		= lirc_read,
@@ -593,6 +578,9 @@ static struct file_operations lirc_fops = {
 	.ioctl		= lirc_ioctl,
 #else
 	.unlocked_ioctl	= lirc_ioctl,
+#ifdef CONFIG_COMPAT
+	.compat_ioctl	= lirc_ioctl,
+#endif
 #endif
 	.open		= lirc_open,
 	.release	= lirc_close
@@ -614,9 +602,6 @@ static struct lirc_driver driver = {
        .sample_rate	= 0,
        .data		= NULL,
        .add_to_buf	= NULL,
-#ifndef LIRC_REMOVE_DURING_EXPORT
-       .get_queue	= NULL,
-#endif
        .set_use_inc	= set_use_inc,
        .set_use_dec	= set_use_dec,
        .fops		= &lirc_fops,
@@ -624,32 +609,9 @@ static struct lirc_driver driver = {
        .owner		= THIS_MODULE,
 };
 
-#ifdef MODULE
 static int pf(void *handle);
 static void kf(void *handle);
 
-static struct timer_list poll_timer;
-static void poll_state(unsigned long ignored);
-
-static void poll_state(unsigned long ignored)
-{
-	printk(KERN_NOTICE "%s: time\n",
-	       LIRC_DRIVER_NAME);
-	del_timer(&poll_timer);
-	if (is_claimed)
-		return;
-	kf(NULL);
-	if (!is_claimed) {
-		printk(KERN_NOTICE "%s: could not claim port, giving up\n",
-		       LIRC_DRIVER_NAME);
-		init_timer(&poll_timer);
-		poll_timer.expires = jiffies + HZ;
-		poll_timer.data = (unsigned long)current;
-		poll_timer.function = poll_state;
-		add_timer(&poll_timer);
-	}
-}
-
 static int pf(void *handle)
 {
 	parport_disable_irq(pport);
@@ -675,16 +637,7 @@ static void kf(void *handle)
 
 static int __init lirc_parallel_init(void)
 {
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 3)
 	pport = parport_find_base(io);
-#else
-	pport = parport_enumerate();
-	while (pport != NULL) {
-		if (pport->base == io)
-			break;
-		pport = pport->next;
-	}
-#endif
 	if (pport == NULL) {
 		printk(KERN_NOTICE "%s: no port at %x found\n",
 		       LIRC_DRIVER_NAME, io);
@@ -692,9 +645,7 @@ static int __init lirc_parallel_init(void)
 	}
 	ppdevice = parport_register_device(pport, LIRC_DRIVER_NAME,
 					   pf, kf, irq_handler, 0, NULL);
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 3)
 	parport_put_port(pport);
-#endif
 	if (ppdevice == NULL) {
 		printk(KERN_NOTICE "%s: parport_register_device() failed\n",
 		       LIRC_DRIVER_NAME);
@@ -766,5 +717,3 @@ MODULE_PARM_DESC(debug, "Enable debugging messages");
 
 module_param(check_pselecd, bool, S_IRUGO | S_IWUSR);
 MODULE_PARM_DESC(debug, "Check for printer (default: 0)");
-EXPORT_NO_SYMBOLS;
-#endif /* MODULE */
