@@ -34,27 +34,14 @@
  */
 
 #include <linux/version.h>
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 18)
-#error "**********************************************************"
-#error " Sorry, this driver needs kernel version 2.2.18 or higher "
-#error "**********************************************************"
-#endif
 #include <linux/module.h>
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33)
 #include <linux/autoconf.h>
-#if !defined(CONFIG_SERIAL_MODULE)
-#if !defined(LIRC_ON_SA1100)
-#warning "******************************************"
-#warning " Your serial port driver is compiled into "
-#warning " the kernel. You will have to release the "
-#warning " port you want to use for LIRC with:      "
-#warning "    setserial /dev/ttySx uart none        "
-#warning "******************************************"
-#endif
 #endif
 
 #include <linux/sched.h>
@@ -73,16 +60,10 @@
 #include <linux/delay.h>
 #include <linux/poll.h>
 #include <asm/system.h>
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
-#include <asm/uaccess.h>
-#endif
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 16)
-#include <asm/io.h>
-#else
 #include <linux/io.h>
-#endif
 #include <asm/irq.h>
 #include <linux/fcntl.h>
+#include <linux/platform_device.h>
 #ifdef LIRC_ON_SA1100
 #include <asm/hardware.h>
 #ifdef CONFIG_SA1100_COLLIE
@@ -93,9 +74,14 @@
 
 #include <linux/timer.h>
 
+#include "drivers/kcompat.h"
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 35)
+#include <media/lirc.h>
+#include <media/lirc_dev.h>
+#else
 #include "drivers/lirc.h"
 #include "drivers/lirc_dev/lirc_dev.h"
-#include "drivers/kcompat.h"
+#endif
 
 /* SECTION: Definitions */
 
@@ -224,8 +210,12 @@ static ssize_t lirc_read(struct file *file, char *buf, size_t count,
 		loff_t *ppos);
 static ssize_t lirc_write(struct file *file, const char *buf, size_t n,
 		loff_t *pos);
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 35)
 static int lirc_ioctl(struct inode *node, struct file *filep, unsigned int cmd,
 		unsigned long arg);
+#else
+static long lirc_ioctl(struct file *filep, unsigned int cmd, unsigned long arg);
+#endif
 static void add_read_queue(int flag, unsigned long val);
 #ifdef MODULE
 static int init_chrdev(void);
@@ -257,12 +247,12 @@ static void off(void)
 	PPSR &= ~PPC_TXD2;
 }
 #else
-static unsigned int sinp(int offset)
+__attribute__ ((unused)) static unsigned int sinp(int offset)
 {
 	return inb(io + offset);
 }
 
-static void soutp(int offset, int value)
+__attribute__ ((unused)) static void soutp(int offset, int value)
 {
 	outb(value, io + offset);
 }
@@ -292,14 +282,12 @@ static int lirc_open(struct inode *inode, struct file *file)
 		spin_unlock(&dev_lock);
 		return -EBUSY;
 	}
-	MOD_INC_USE_COUNT;
 	spin_unlock(&dev_lock);
 	return 0;
 }
 
 static int lirc_close(struct inode *inode, struct file *file)
 {
-	MOD_DEC_USE_COUNT;
 	return 0;
 }
 
@@ -392,13 +380,16 @@ static ssize_t lirc_write(struct file *file, const char *buf, size_t n,
 	return n;
 }
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 35)
 static int lirc_ioctl(struct inode *node, struct file *filep, unsigned int cmd,
 		unsigned long arg)
+#else
+static long lirc_ioctl(struct file *filep, unsigned int cmd, unsigned long arg)
+#endif
 {
 	int retval = 0;
-	unsigned long value = 0;
+	__u32 value = 0;
 #ifdef LIRC_ON_SA1100
-	unsigned int ivalue;
 
 	if (cmd == LIRC_GET_FEATURES)
 		value = LIRC_CAN_SEND_PULSE |
@@ -422,22 +413,22 @@ static int lirc_ioctl(struct inode *node, struct file *filep, unsigned int cmd,
 	case LIRC_GET_FEATURES:
 	case LIRC_GET_SEND_MODE:
 	case LIRC_GET_REC_MODE:
-		retval = put_user(value, (unsigned long *) arg);
+		retval = put_user(value, (__u32 *) arg);
 		break;
 
 	case LIRC_SET_SEND_MODE:
 	case LIRC_SET_REC_MODE:
-		retval = get_user(value, (unsigned long *) arg);
+		retval = get_user(value, (__u32 *) arg);
 		break;
 #ifdef LIRC_ON_SA1100
 	case LIRC_SET_SEND_DUTY_CYCLE:
-		retval = get_user(ivalue, (unsigned int *) arg);
+		retval = get_user(value, (__u32 *) arg);
 		if (retval)
-			return reetval;
-		if (ivalue <= 0 || ivalue > 100)
+			return retval;
+		if (value <= 0 || value > 100)
 			return -EINVAL;
-		/* (ivalue/100)*(1000000/freq) */
-		duty_cycle = ivalue;
+		/* (value/100)*(1000000/freq) */
+		duty_cycle = value;
 		pulse_width = (unsigned long) duty_cycle*10000/freq;
 		space_width = (unsigned long) 1000000L/freq-pulse_width;
 		if (pulse_width >= LIRC_ON_SA1100_TRANSMITTER_LATENCY)
@@ -446,12 +437,12 @@ static int lirc_ioctl(struct inode *node, struct file *filep, unsigned int cmd,
 			space_width -= LIRC_ON_SA1100_TRANSMITTER_LATENCY;
 		break;
 	case LIRC_SET_SEND_CARRIER:
-		retval = get_user(ivalue, (unsigned int *) arg);
+		retval = get_user(value, (__u32 *) arg);
 		if (retval)
 			return retval;
-		if (ivalue > 500000 || ivalue < 20000)
+		if (value > 500000 || value < 20000)
 			return -EINVAL;
-		freq = ivalue;
+		freq = value;
 		pulse_width = (unsigned long) duty_cycle*10000/freq;
 		space_width = (unsigned long) 1000000L/freq-pulse_width;
 		if (pulse_width >= LIRC_ON_SA1100_TRANSMITTER_LATENCY)
@@ -461,7 +452,11 @@ static int lirc_ioctl(struct inode *node, struct file *filep, unsigned int cmd,
 		break;
 #endif
 	default:
-		retval = -ENOIOCTLCMD;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 35)
+		retval = lirc_dev_fop_ioctl(node, filep, cmd, arg);
+#else
+		retval = lirc_dev_fop_ioctl(filep, cmd, arg);
+#endif
 
 	}
 
@@ -516,7 +511,14 @@ static struct file_operations lirc_fops = {
 	.read		= lirc_read,
 	.write		= lirc_write,
 	.poll		= lirc_poll,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 35)
 	.ioctl		= lirc_ioctl,
+#else
+	.unlocked_ioctl	= lirc_ioctl,
+#ifdef CONFIG_COMPAT
+	.compat_ioctl	= lirc_ioctl,
+#endif
+#endif
 	.open		= lirc_open,
 	.release	= lirc_close,
 };
@@ -537,9 +539,6 @@ static struct lirc_driver driver = {
        .sample_rate	= 0,
        .data		= NULL,
        .add_to_buf	= NULL,
-#ifndef LIRC_REMOVE_DURING_EXPORT
-       .get_queue	= NULL,
-#endif
        .set_use_inc	= set_use_inc,
        .set_use_dec	= set_use_dec,
        .fops		= &lirc_fops,
@@ -547,10 +546,12 @@ static struct lirc_driver driver = {
        .owner		= THIS_MODULE,
 };
 
+static struct platform_device *lirc_sir_dev;
 
 #ifdef MODULE
 static int init_chrdev(void)
 {
+	driver.dev = &lirc_sir_dev->dev;
 	driver.minor = lirc_register_driver(&driver);
 	if (driver.minor < 0) {
 		printk(KERN_ERR LIRC_DRIVER_NAME ": init_chrdev() failed.\n");
@@ -1289,21 +1290,73 @@ static int init_lirc_sir(void)
 	return 0;
 }
 
+static int __devinit lirc_sir_probe(struct platform_device *dev)
+{
+	return 0;
+}
+
+static int __devexit lirc_sir_remove(struct platform_device *dev)
+{
+	return 0;
+}
+
+static struct platform_driver lirc_sir_driver = {
+	.probe		= lirc_sir_probe,
+	.remove		= __devexit_p(lirc_sir_remove),
+	.driver		=	{
+		.name	= "lirc_sir",
+		.owner	= THIS_MODULE,
+	},
+};
+
 #ifdef MODULE
 
 static int __init lirc_sir_init(void)
 {
 	int retval;
 
+	retval = platform_driver_register(&lirc_sir_driver);
+	if (retval) {
+		printk(KERN_ERR LIRC_DRIVER_NAME ": Platform driver register "
+		       "failed!\n");
+		return -ENODEV;
+	}
+
+	lirc_sir_dev = platform_device_alloc("lirc_dev", 0);
+	if (!lirc_sir_dev) {
+		printk(KERN_ERR LIRC_DRIVER_NAME ": Platform device alloc "
+		       "failed!\n");
+		retval = -ENOMEM;
+		goto pdev_alloc_fail;
+	}
+
+	retval = platform_device_add(lirc_sir_dev);
+	if (retval) {
+		printk(KERN_ERR LIRC_DRIVER_NAME ": Platform device add "
+		       "failed!\n");
+		retval = -ENODEV;
+		goto pdev_add_fail;
+	}
+
 	retval = init_chrdev();
 	if (retval < 0)
-		return retval;
+		goto fail;
+
 	retval = init_lirc_sir();
 	if (retval) {
 		drop_chrdev();
-		return retval;
+		goto fail;
 	}
+
 	return 0;
+
+fail:
+	platform_device_del(lirc_sir_dev);
+pdev_add_fail:
+	platform_device_put(lirc_sir_dev);
+pdev_alloc_fail:
+	platform_driver_unregister(&lirc_sir_driver);
+	return retval;
 }
 
 static void __exit lirc_sir_exit(void)
@@ -1311,6 +1364,8 @@ static void __exit lirc_sir_exit(void)
 	drop_hardware();
 	drop_chrdev();
 	drop_port();
+	platform_device_unregister(lirc_sir_dev);
+	platform_driver_unregister(&lirc_sir_driver);
 	printk(KERN_INFO LIRC_DRIVER_NAME ": Uninstalled.\n");
 }
 
@@ -1352,6 +1407,4 @@ MODULE_PARM_DESC(threshold, "space detection threshold (3)");
 module_param(debug, bool, S_IRUGO | S_IWUSR);
 MODULE_PARM_DESC(debug, "Enable debugging messages");
 
-EXPORT_NO_SYMBOLS;
-
 #endif /* MODULE */
