@@ -31,11 +31,6 @@
 #endif
 
 #include <linux/version.h>
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 18)
-#error "**********************************************************"
-#error " Sorry, this driver needs kernel version 2.2.18 or higher "
-#error "**********************************************************"
-#endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33)
 #include <linux/autoconf.h>
@@ -53,22 +48,21 @@
 
 #include <linux/bitops.h>
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 16)
-#include <asm/io.h>
-#else
 #include <linux/io.h>
-#endif
 #include <linux/irq.h>
 
 #include <linux/acpi.h>
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
 #include <linux/platform_device.h>
-#endif
 
-#include "drivers/lirc.h"
 #include "drivers/kcompat.h"
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 35)
+#include <media/lirc.h>
+#include <media/lirc_dev.h>
+#else
+#include "drivers/lirc.h"
 #include "drivers/lirc_dev/lirc_dev.h"
+#endif
 
 #include "lirc_wpc8769l.h"
 
@@ -851,7 +845,6 @@ static int set_use_inc(void *data)
 	/* Enable hardware interrupts. */
 	wpc8769l_enable_interrupts();
 
-	MOD_INC_USE_COUNT;
 	return 0;
 }
 
@@ -873,8 +866,6 @@ static void set_use_dec(void *data)
 
 	/* Free the RX buffer. */
 	lirc_buffer_free(&rbuf);
-
-	MOD_DEC_USE_COUNT;
 }
 
 static struct lirc_driver driver = {
@@ -884,7 +875,6 @@ static struct lirc_driver driver = {
 	.sample_rate	= 0,
 	.data		= NULL,
 	.add_to_buf	= NULL,
-	.get_queue	= NULL,
 	.rbuf		= &rbuf,
 	.set_use_inc	= set_use_inc,
 	.set_use_dec	= set_use_dec,
@@ -938,7 +928,6 @@ static int wpc8769l_acpi_detect(void)
 }
 
 #ifdef MODULE
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
 static struct platform_device *lirc_wpc8769l_platform_dev;
 
 static int __devinit lirc_wpc8769l_probe(struct platform_device *dev)
@@ -1018,7 +1007,6 @@ static void __exit lirc_wpc8769l_platform_exit(void)
 	platform_device_unregister(lirc_wpc8769l_platform_dev);
 	platform_driver_unregister(&lirc_wpc8769l_platform_driver);
 }
-#endif
 
 static int __init lirc_wpc8769l_module_init(void)
 {
@@ -1066,12 +1054,10 @@ static int __init lirc_wpc8769l_module_init(void)
 		}
 	}
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
 	/* Register the platform driver and device. */
 	rc = lirc_wpc8769l_platform_init();
 	if (rc)
 		goto exit_release_region_2;
-#endif
 
 	/* Prepare the hardware. */
 	wpc8769l_prepare_hardware();
@@ -1093,11 +1079,9 @@ static int __init lirc_wpc8769l_module_init(void)
 	return 0; /* Everything OK. */
 
 exit_platform_exit:
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
 	lirc_wpc8769l_platform_exit();
 
 exit_release_region_2:
-#endif
 	if (baseport2)
 		release_region(baseport2, WPC8769L_IO_REGION_2_SIZE);
 
@@ -1111,10 +1095,8 @@ module_init(lirc_wpc8769l_module_init);
 
 static void __exit lirc_wpc8769l_module_exit(void)
 {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
 	/* Unregister the platform driver and device. */
 	lirc_wpc8769l_platform_exit();
-#endif
 
 	/* Unregister the LIRC driver. */
 	lirc_unregister_driver(driver.minor);
@@ -1175,7 +1157,5 @@ MODULE_PARM_DESC(rc_wakeup_mask,
 	"Define the RC code mask for wake up functions (default: 0xff000fff).");
 #endif
 
-EXPORT_NO_SYMBOLS;
-
 #endif /* MODULE */
 
